/* Playlist.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "Playlist.h"
#include <QDir>

Playlist::Playlist(int idx, QString name) :
	PlaylistDBInterface(name),
	SayonaraClass()
{
	_playlist_changed = false;
	_cur_play_idx = -1;
	_playlist_idx = idx;
	_playlist_type = Playlist::Type::Std;
	_playlist_mode = _settings->get(Set::PL_Mode);

}

Playlist::~Playlist(){
}

void Playlist::clear() {
	if(_v_md.isEmpty()){
		return;
	}

	if(!this->is_temporary()){
		_playlist_changed = true;
	}
    _v_md.clear();
}

void Playlist::move_track(const int idx, int tgt) {
	if(!this->is_temporary()){
		_playlist_changed = true;
	}
	IdxList lst;
    lst << idx;
    move_tracks(lst, tgt);
}

void Playlist::move_tracks(const IdxList& lst, int tgt) {
	if(!this->is_temporary()){
		_playlist_changed = true;
	}
	_v_md.move_tracks(lst, tgt);
}


void Playlist::delete_track(const int idx) {
	IdxList to_remove;
    to_remove << idx;
    delete_tracks(to_remove);
}

void Playlist::delete_tracks(const IntList& lst) {
	if(!this->is_temporary()){
		_playlist_changed = true;
	}
	_v_md.remove_tracks(lst);
}


void Playlist::insert_track(const MetaData& md, int tgt) {
    MetaDataList v_md;
	v_md << md;
    insert_tracks(v_md, tgt);
}


void Playlist::insert_tracks(const MetaDataList& lst, int tgt) {
	if(!this->is_temporary()){
		_playlist_changed = true;
	}
	_v_md.insert_tracks(lst, tgt);
}

void Playlist::selection_changed(const IdxList& lst) {

	_selected_tracks.clear();

	for(MetaData& md : _v_md){
		md.pl_selected = false;
	}

	for(int idx : lst){
		_v_md[idx].pl_selected = true;
		_selected_tracks << _v_md[idx];
	}
}


void Playlist::append_track(const MetaData& md) {

    MetaDataList v_md;
	v_md << md;
    append_tracks(v_md);
}

void Playlist::append_tracks(const MetaDataList& lst) {
	if(!this->is_temporary()){
		_playlist_changed = true;
	}
	for(const MetaData& md : lst){
		_v_md << std::move(md);
		_v_md.last().is_disabled = !(Helper::check_track(md));
    }
}


void Playlist::replace_track(int idx, const MetaData& md) {

    if(idx < 0 || idx >= _v_md.size()) return;

	bool is_selected = _v_md[idx].pl_selected;
	bool is_playing = _v_md[idx].pl_playing;

    _v_md[idx] = md;
    _v_md[idx].is_disabled = !(Helper::check_track(md));
	_v_md[idx].pl_selected = is_selected;
	_v_md[idx].pl_playing = is_playing;
}


Playlist::Type Playlist::get_type() const {
    return _playlist_type;
}

int Playlist::get_idx() const {
	return _playlist_idx;
}

void Playlist::set_idx(int idx){
	_playlist_idx = idx;
}

void Playlist::set_playlist_mode(const PlaylistMode& mode) {

	if(_playlist_mode.shuffle != mode.shuffle){
		for(MetaData& md : _v_md){
			md.played = false;
		}
	}

    _playlist_mode = mode;
}

PlaylistMode Playlist::get_playlist_mode() const {
	return _playlist_mode;
}

int Playlist::get_cur_track_idx() const {
	return _v_md.get_cur_play_track();
}

bool Playlist::get_cur_track(MetaData &md) const {
	int cur_play_idx = _v_md.get_cur_play_track();

	if(cur_play_idx < 0){
		return false;
	}

	md = _v_md[cur_play_idx];
	return true;
}

bool Playlist::is_empty() const {

    return (_v_md.size() == 0);
}


QStringList Playlist::toStringList() const {
    return _v_md.toStringList();
}

IdxList Playlist::find_tracks(int idx) const {
	return _v_md.findTracks(idx);
}

IdxList Playlist::find_tracks(const QString& filepath) const {
	return _v_md.findTracks(filepath);
}

int Playlist::get_size() const {
	return _v_md.size();
}

MetaDataList Playlist::get_playlist() const
{
	return _v_md;
}

MetaDataList Playlist::get_selected_tracks() const
{
	return _selected_tracks;
}

void Playlist::set_changed(bool b){
	_playlist_changed = b;
}

bool Playlist::was_changed() const
{
	return _playlist_changed;
}
