/* GUI_Playlist.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * GUI_Playlist.cpp
 *
 *  Created on: Apr 6, 2011
 *      Author: luke
 */

#include "GUI_Playlist.h"

#include <QKeyEvent>
#include <QFileDialog>
#include <QScrollBar>
#include <QTextEdit>
#include <QAction>
#include <QMenu>

GUI_Playlist::GUI_Playlist(QWidget *parent) :
	SayonaraWidget(parent),
	Ui::Playlist_Window()
{
	setupUi(this);

	_playlist = PlaylistHandler::getInstance();
	_play_manager = PlayManager::getInstance();
	_info_dialog = new GUI_InfoDialog(this);

	tw_playlists->removeTab(0);

    QAction* clear_action = new QAction(this);
    clear_action->setShortcut(QKeySequence(tr("Ctrl+.")));
    clear_action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
	connect(clear_action, &QAction::triggered, btn_clear, &QPushButton::click);
	addAction(clear_action);

	bottom_bar->check_dynamic_play_button();
    setAcceptDrops(true);


	connect(_playlist,	&PlaylistHandler::sig_cur_track_idx_changed,
			this,		&GUI_Playlist::playlist_track_changed);
	connect(_playlist,	&PlaylistHandler::sig_playlist_created,
			this,		&GUI_Playlist::playlist_fill);
	connect(_playlist,	&PlaylistHandler::sig_playlist_name_changed,
			this,		&GUI_Playlist::playlist_name_changed);
	connect(_playlist,	&PlaylistHandler::sig_new_playlist_added,
			this,		&GUI_Playlist::playlist_added);

	connect(_play_manager, &PlayManager::sig_playlist_finished,	this, &GUI_Playlist::playlist_finished);
	connect(_play_manager, &PlayManager::sig_playstate_changed,	this, &GUI_Playlist::playstate_changed);

	connect(btn_clear, &QPushButton::clicked, this, &GUI_Playlist::clear_button_pressed);
	connect(btn_new_tab, &QPushButton::clicked, this, &GUI_Playlist::add_playlist_button_pressed);

	connect(tw_playlists, &PlaylistTabWidget::tabCloseRequested, this, &GUI_Playlist::tab_close_playlist_clicked);
	connect(tw_playlists, &PlaylistTabWidget::currentChanged, this, &GUI_Playlist::tab_playlist_clicked);
	connect(tw_playlists, &PlaylistTabWidget::sig_tab_delete, this, &GUI_Playlist::tab_delete_playlist_clicked);
	connect(tw_playlists, &PlaylistTabWidget::sig_tab_save, this, &GUI_Playlist::tab_save_playlist_clicked);
	connect(tw_playlists, &PlaylistTabWidget::sig_tab_save_as, this, &GUI_Playlist::tab_save_playlist_as_clicked);
	connect(tw_playlists, &PlaylistTabWidget::sig_tab_rename, this, &GUI_Playlist::tab_rename_clicked);
	connect(tw_playlists, &PlaylistTabWidget::sig_tab_reset, this, &GUI_Playlist::tab_reset_clicked);

	init_playlist_view(listView);

	REGISTER_LISTENER(Set::PL_SmallItems, _sl_small_playlist_items_changed);
	REGISTER_LISTENER(Set::Lib_Path, _sl_library_path_changed);
	REGISTER_LISTENER(Set::PL_ShowNumbers, _sl_show_numbers_changed);

	_playlist->load_old_playlists();

	tw_playlists->setCurrentIndex(_playlist->get_current_idx());
	tw_playlists->setTabsClosable(_playlist_views.size() > 1);
}


GUI_Playlist::~GUI_Playlist() {
	delete _info_dialog;
}

void GUI_Playlist::changeEvent(QEvent* e) {
    e->accept();
}

void GUI_Playlist::resizeEvent(QResizeEvent* e) {

	PlaylistView* cur_view = get_current_view();

    e->accept();

	if(cur_view){
		cur_view->update();
		cur_view->reset();
	}
}

void GUI_Playlist::language_changed() {

	retranslateUi(this);
	set_total_time_label( _total_time[ tw_playlists->currentIndex()] );

	bottom_bar->check_dynamic_play_button();
}

void GUI_Playlist::init_playlist_view(const PlaylistView* pl_view){

	connect(pl_view, &PlaylistView::sig_metadata_dropped, this, &GUI_Playlist::metadata_dropped);
	connect(pl_view, &PlaylistView::sig_rows_removed, this, &GUI_Playlist::rows_removed);
	connect(pl_view, &PlaylistView::sig_rows_moved, this, &GUI_Playlist::rows_moved);
	connect(pl_view, &PlaylistView::sig_info_clicked, this, &GUI_Playlist::menu_info_clicked);
	connect(pl_view, &PlaylistView::sig_edit_clicked, this, &GUI_Playlist::menu_edit_clicked);
	connect(pl_view, &PlaylistView::sig_sel_changed,this, &GUI_Playlist::sel_changed);
	connect(pl_view, &PlaylistView::sig_double_clicked, this, &GUI_Playlist::double_clicked);
}


void GUI_Playlist::playlist_track_changed(int row, int playlist_idx) {

	tab_clear_all_icons();

	if( !between(playlist_idx, 0, _playlist_views.size())) {
		return;
	}

	int n_rows;

	_playlist_views[playlist_idx]->set_current_track(row);
	n_rows = _playlist_views[playlist_idx]->get_num_rows();

	if(n_rows > 0){
		QIcon icon_play = Helper::get_icon("play_bordered");
		tw_playlists->setTabIcon(playlist_idx, icon_play);
	}
}


void GUI_Playlist::playlist_fill(const PlaylistPtr& pl) {

	int pl_idx;
	qint64 dur_ms;
	bool temporary, was_changed;

	pl_idx = pl->get_idx();
	if( !between(pl_idx, 0, _playlist_views.size())) {
		return;
	}

	const MetaDataList& v_md = pl->get_playlist();
	temporary = pl->is_temporary();
	was_changed = pl->was_changed();

	_playlist_views[pl_idx]->fill( v_md );

	if(was_changed && !temporary){
		tw_playlists->setTabText(pl_idx, QString("* ") + pl->get_name());
	}
	else{
		tw_playlists->setTabText(pl_idx, pl->get_name());
	}

	tw_playlists->show_menu_items(!temporary,	// save
								  true,			// save_as
								  !temporary,	// delete
								  (!temporary && was_changed)	// reset
	);

	dur_ms = 0;
	for(const MetaData& md : v_md) {
		dur_ms += md.length_ms;
    }

	_total_time[pl_idx] = dur_ms;
	set_total_time_label(dur_ms);
}


void GUI_Playlist::clear_button_pressed() {

	PlaylistView* cur_view = get_current_view();

	if(cur_view){
		 int cur_idx = tw_playlists->currentIndex();
		 cur_view->clear();
		_total_time[cur_idx] = 0;
	}

	_playlist->clear_playlist();
	lab_totalTime->setText(tr("Playlist empty"));
}


void GUI_Playlist::add_playlist_button_pressed(){
	QString name = _playlist->request_new_playlist_name();
	_playlist->create_empty_playlist(name);
}


void GUI_Playlist::metadata_dropped(const MetaDataList& v_md, int row) {
	_playlist->insert_tracks(v_md, row, _playlist->get_current_idx());
}


void GUI_Playlist::rows_moved(const IdxList& lst, int tgt_idx) {

	if(lst.isEmpty()){
		return;
	}

	_playlist->move_rows(lst, tgt_idx, _playlist->get_current_idx());
}


void GUI_Playlist::rows_removed(const IdxList& lst) {
	_playlist->remove_rows(lst, _playlist->get_current_idx());
}


void GUI_Playlist::double_clicked(int row) {

	int cur_idx = tw_playlists->currentIndex();
	_playlist->change_track(row, cur_idx);
}


void GUI_Playlist::sel_changed(const MetaDataList& v_md, const IdxList& sel_rows) {

	if(sel_rows.isEmpty()){
		return;
	}

	_info_dialog->set_metadata(v_md, GUI_InfoDialog::Mode::Tracks);

	_playlist->selection_changed(sel_rows);
}


void GUI_Playlist::menu_info_clicked() {
    if(!_info_dialog) return;
	_info_dialog->show(GUI_InfoDialog::TabInfo);
}


void GUI_Playlist::menu_edit_clicked() {
    if(!_info_dialog) return;
	_info_dialog->show(GUI_InfoDialog::TabInfo);
}


void GUI_Playlist::dragLeaveEvent(QDragLeaveEvent* event) {
    event->accept();
}


void GUI_Playlist::dragEnterEvent(QDragEnterEvent* event) {
    event->accept();
}


void GUI_Playlist::dragMoveEvent(QDragMoveEvent* event) {
	PlaylistView* cur_view = get_current_view();

	if(!cur_view) return;

	if( event->pos().y() < cur_view->y()){
		cur_view->scroll_up();
	}

	else if(event->pos().y() > cur_view->y() + cur_view->height()){
		cur_view->scroll_down();
	}
}


void GUI_Playlist::dropEvent(QDropEvent* event) {
	PlaylistView* cur_view = get_current_view();
	if(cur_view){
		cur_view->dropEventFromOutside(event);
	}
}

void GUI_Playlist::set_total_time_label(qint64 dur_ms) {

	PlaylistView* cur_view = get_current_view();

	int n_rows = 0;
	QString text;
	QString time_str;
	QString playlist_string;

	if(dur_ms > 0){
		time_str = Helper::cvt_ms_to_string(this, dur_ms, true, false);
	}

	if(cur_view){
		n_rows = cur_view->get_num_rows();
	}

	playlist_string = text + QString::number(n_rows);

	if(n_rows == 1)	{
		playlist_string += " " + tr("Track");
	}

	else {
		playlist_string += " " + tr("Tracks");
	}

	if( !time_str.isEmpty() ){
		playlist_string += " - " + time_str;
	}

	lab_totalTime->setText(playlist_string);
	lab_totalTime->setContentsMargins(0, 2, 0, 2);
}



void GUI_Playlist::set_playlist_type(Playlist::Type playlist_type) {

	int actions = 0;
	PlaylistView* cur_view = get_current_view();

	bottom_bar->set_playlist_type(playlist_type);

	if(playlist_type != Playlist::Type::Std){
        actions = ENTRY_INFO;
    }

    else{
        actions = (ENTRY_INFO | ENTRY_REMOVE | ENTRY_EDIT);
    }

	if(cur_view){
		cur_view->set_context_menu_actions(actions);
	}
}

void GUI_Playlist::_sl_show_numbers_changed(){

	PlaylistView* cur_view = get_current_view();

	parentWidget()->setFocus();

	if( cur_view ){
	   cur_view->reset();
	}
}


void GUI_Playlist::_sl_library_path_changed() {

	bottom_bar->check_dynamic_play_button();
}

void GUI_Playlist::_sl_small_playlist_items_changed() {

	PlaylistView* cur_view = get_current_view();
	bool small_items = _settings->get(Set::PL_SmallItems);

	if(cur_view) {
		cur_view->show_big_items(!small_items);
	}
}
