/* LibraryPluginHandler.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "LibraryPluginHandler.h"
#include "GUI/Helper/ComboBoxDelegate/ComboBoxDelegate.h"

#include <QLayout>

LibraryPluginHandler::LibraryPluginHandler(QObject* parent) :
	QObject(parent),
	SayonaraClass()
{

	_cur_idx = -1;
	REGISTER_LISTENER(Set::Player_Language, language_changed);
}

void LibraryPluginHandler::init(const QList<LibraryContainerInterface*>& containers)
{

	QString lib_dir;
	QList<QPair<QString, QIcon>> library_entries;
	QDir plugin_dir;
	QStringList dll_filenames;
	QString cur_plugin;

	lib_dir = Helper::get_lib_path();
	plugin_dir = QDir(lib_dir);
	dll_filenames = plugin_dir.entryList(QDir::Files);
	cur_plugin = _settings->get(Set::Lib_CurPlugin);


	for(LibraryContainerInterface* container : containers){

		if(!container){
			continue;
		}

		QString display_name = container->get_display_name();
		QIcon icon = container->get_icon();

		library_entries << QPair<QString, QIcon>(display_name, icon);

		sp_log(Log::Debug) << "Add plugin " << container->get_display_name();

		_libraries << container;
	}

	for(const QString& filename : dll_filenames) {

		QString display_name;
		QIcon icon;
		QObject* raw_plugin;
		LibraryContainerInterface* container;

		QPluginLoader loader(plugin_dir.absoluteFilePath(filename));

		raw_plugin = loader.instance();
		if(!raw_plugin) {
			sp_log(Log::Warning) << "Cannot load plugin: " << filename << ": " << loader.errorString();
			continue;
		}

		container = dynamic_cast<LibraryContainerInterface*>(raw_plugin);
		if(!container) {
			continue;
		}

		sp_log(Log::Info) << "Found library plugin " << container->get_display_name();
		_libraries << container;

		display_name = container->get_display_name();
		icon = container->get_icon();

		library_entries << QPair<QString, QIcon>(display_name, icon);
	}

	sp_log(Log::Info) << "Found " << _libraries.size() << " library types";

	int i=0;
	for(LibraryContainerInterface* container : _libraries ){
		QWidget* ui;
		QLayout* layout;
		QComboBox* libchooser;

		ui = container->get_ui();
		layout = ui->layout();
		if(layout){
			layout->setContentsMargins(5, 0, 8, 0);
		}

		if(cur_plugin == container->get_name()){
			_cur_idx = i;
		}

		libchooser = container->get_libchooser();
		libchooser->setIconSize(QSize(16,16));
		libchooser->setMinimumWidth(200);
		libchooser->setMaximumWidth(200);
		libchooser->setMinimumHeight(25);
		libchooser->setMaximumHeight(28);
		libchooser->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);

		libchooser->setItemDelegate(new ComboBoxDelegate(ui));

		for(const QPair<QString, QIcon>& entry : library_entries){
			libchooser->addItem(entry.second, entry.first);
		}

		libchooser->setCurrentIndex(i);

		connect(libchooser, combo_activated_int, this, &LibraryPluginHandler::index_changed);
		i++;
	}
}



LibraryPluginHandler::~LibraryPluginHandler()
{

}

void LibraryPluginHandler::index_changed(int idx){

	_cur_idx = idx;
	int i=0;
	for(LibraryContainerInterface* container : _libraries){

		QWidget* ui, *parent;
		QString name;

		ui = container->get_ui();
		parent = ui->parentWidget();
		name = container->get_display_name();

		if(i == idx){
			ui->setVisible(true);

			container->get_libchooser()->setCurrentIndex(i);

			if(parent){
				ui->resize(parent->size());
			}

			ui->update();

			_settings->set(Set::Lib_CurPlugin, container->get_name());
		}

		else{
			ui->setVisible(false);
		}

		i++;
	}
}

LibraryContainerInterface* LibraryPluginHandler::get_cur_library() const {
	if(!between(_cur_idx, 0, _libraries.size())) {
		return nullptr;
	}

	return _libraries[_cur_idx];
}

void LibraryPluginHandler::language_changed(){


	for(LibraryContainerInterface* container : _libraries){

		QComboBox* libchooser;
		libchooser = container->get_libchooser();
		int i=0;

		for(LibraryContainerInterface* container2 : _libraries){
			libchooser->setItemText(i, container2->get_display_name());
			i++;
		}
	}
}


QList<LibraryContainerInterface*> LibraryPluginHandler::get_libraries() const {
	return _libraries;
}

